@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-encodings-dialog.c 4429 2005-12-12 17:28:04Z pborelli $
+ * $Id: gedit-encodings-dialog.c 6379 2008-08-10 16:40:21Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -300,6 +300,10 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 	GtkWidget *error_widget;
 	int i;
 	gboolean ret;
+	gchar *root_objects[] = {
+		"encodings-dialog-contents",
+		NULL
+	};
 
 	dlg->priv = GEDIT_ENCODINGS_DIALOG_GET_PRIVATE (dlg);
 	
@@ -315,6 +319,12 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 	gtk_window_set_title (GTK_WINDOW (dlg), _("Character Codings"));
 	gtk_window_set_default_size (GTK_WINDOW (dlg), 650, 400);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
+	
+	/* HIG defaults */
+	gtk_container_set_border_width (GTK_CONTAINER (dlg), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 2); /* 2 * 5 + 2 = 12 */
+	gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (dlg)->action_area), 5);
+	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->action_area), 6);	
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dlg),
 					 GTK_RESPONSE_OK);
@@ -324,15 +334,15 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 			  G_CALLBACK (response_handler),
 			  dlg);
 
-	ret = gedit_utils_get_glade_widgets (GEDIT_GLADEDIR "gedit-encodings-dialog.glade",
-					     "encodings-dialog-contents",
-					     &error_widget,
-					     "encodings-dialog-contents", &content,
-					     "add-button", &dlg->priv->add_button,
-					     "remove-button", &dlg->priv->remove_button,
-					     "available-treeview", &dlg->priv->available_treeview,
-					     "displayed-treeview", &dlg->priv->displayed_treeview,
-					     NULL);
+	ret = gedit_utils_get_ui_objects (GEDIT_UIDIR "gedit-encodings-dialog.ui",
+					  root_objects,
+					  &error_widget,
+					  "encodings-dialog-contents", &content,
+					  "add-button", &dlg->priv->add_button,
+					  "remove-button", &dlg->priv->remove_button,
+					  "available-treeview", &dlg->priv->available_treeview,
+					  "displayed-treeview", &dlg->priv->displayed_treeview,
+					  NULL);
 
 	if (!ret)
 	{
@@ -340,12 +350,15 @@ gedit_encodings_dialog_init (GeditEncodingsDialog *dlg)
 
 		gtk_box_pack_start_defaults (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 					     error_widget);
+		gtk_container_set_border_width (GTK_CONTAINER (error_widget), 5);			     
 
 		return;
 	}
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dlg)->vbox),
 			    content, TRUE, TRUE, 0);
+	g_object_unref (content);
+	gtk_container_set_border_width (GTK_CONTAINER (content), 5);			     
 
 	g_signal_connect (dlg->priv->add_button,
 			  "clicked",
