@@ -174,7 +174,6 @@ async_failed (AsyncData *async,
 	      GError    *error)
 {
 	g_propagate_error (&async->saver->priv->error, error);
-	g_error_free (error);
 	
 	remote_save_completed_or_failed (async->saver, async);
 }
@@ -237,7 +236,6 @@ remote_reget_info_cb (GFile        *source,
 	{
 		gedit_debug_message (DEBUG_SAVER, "Query info failed: %s", error->message);
 		g_propagate_error (&saver->priv->error, error);
-		g_error_free (error);
 	}
 
 	remote_save_completed_or_failed (saver, async);
@@ -294,7 +292,6 @@ close_async_ready_cb (GOutputStream *stream,
 	if (!g_output_stream_close_finish (stream, res, &error))
 	{
 		g_propagate_error (&async->saver->priv->error, error);
-		g_error_free (error);
 	}
 
 	remote_save_completed_or_failed (async->saver, async);
@@ -340,7 +337,6 @@ remote_get_info_cb (GFileOutputStream *stream,
 		{
 			gedit_debug_message (DEBUG_SAVER, "Query info failed: %s", error->message);
 			g_propagate_error (&saver->priv->error, error);
-			g_error_free (error);
 
 			next_callback = (GAsyncReadyCallback) close_async_ready_cb;
 		}
@@ -621,7 +617,7 @@ check_modification_callback (GFile        *source,
 			     AsyncData    *async)
 {
 	GeditGioDocumentSaver *gvsaver;
-	GError *error;
+	GError *error = NULL;
 	GFileInfo *info;
 	
 	/* manually check cancelled state */
@@ -642,14 +638,18 @@ check_modification_callback (GFile        *source,
 			return;
 		}
 		
-		gedit_debug_message (DEBUG_SAVER, "Error getting modification: %s", error->message);
+		/* it's perfectly fine if the file doesn't exist yet */
+		if (error->code != G_IO_ERROR_NOT_FOUND)
+		{
+			gedit_debug_message (DEBUG_SAVER, "Error getting modification: %s", error->message);
 
-		async_failed (async, error);
-		return;
+			async_failed (async, error);
+			return;
+		}
 	}
 
 	/* check if the mtime is > what we know about it (if we have it) */
-	if (g_file_info_has_attribute (info,
+	if (info != NULL && g_file_info_has_attribute (info,
 				       G_FILE_ATTRIBUTE_TIME_MODIFIED))
 	{
 		GTimeVal mtime;
@@ -673,7 +673,8 @@ check_modification_callback (GFile        *source,
 		}
 	}
 
-	g_object_unref (info);
+	if (info != NULL)
+		g_object_unref (info);
 
 	/* modification check passed, start write */
 	begin_write (async);
