@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit.c 6398 2008-08-13 16:12:21Z johan $
+ * $Id: gedit.c 6516 2008-09-08 18:15:20Z pborelli $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -449,6 +449,10 @@ main (int argc, char *argv[])
 	gboolean restored = FALSE;
 	GError *error = NULL;
 
+	/* Init glib threads asap */
+	g_thread_init (NULL);
+	gdk_threads_init ();
+
 	/* Setup debugging */
 	gedit_debug_init ();
 	gedit_debug_message (DEBUG_APP, "Startup");
@@ -466,13 +470,13 @@ main (int argc, char *argv[])
 	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
 	g_option_context_add_group (context, gtk_get_option_group (FALSE));
 	g_option_context_add_group (context, egg_sm_client_get_option_group ());
-	
-	g_thread_init (NULL);
+
 	gtk_init (&argc, &argv);
 
 	if (!g_option_context_parse (context, &argc, &argv, &error))
 	{
-	        g_print(_("%s\nRun '%s --help' to see a full list of available command line options.\n"), error->message, argv[0]);
+	        g_print(_("%s\nRun '%s --help' to see a full list of available command line options.\n"),
+			error->message, argv[0]);
 		g_error_free (error);
 		return 1;
 	}
@@ -510,7 +514,9 @@ main (int argc, char *argv[])
 		}
 	}
 	else
+	{
 		g_warning ("Cannot create the 'gedit' connection.");
+	}
 
 	gedit_debug_message (DEBUG_APP, "Set icon");
 	
@@ -589,7 +595,6 @@ main (int argc, char *argv[])
 	gedit_prefs_manager_app_shutdown ();
 	gedit_metadata_manager_shutdown ();
 
-
 	return 0;
 }
 
